fig813 <- function(post=F)
{#The first part is the that in fig812.s
 if(post)
  {#ps.options (colors=ps.colors.rgb[c("blue", "magenta", "cyan",
   #"green", "MediumBlue", "red", "black", "yellow", "blue"),])
    postscript("Fig813.ps", width=4.4, height=4.5, horizontal=F,
	pointsize=8)
  }
 else {if(dev.cur()==1) motif(); par(ask=T)}

 par(mfrow = c(2,1), mar=c(5,3,3,1)+0.1)
 par(mex=0.5,csi=0.11)
 
 SP500 <- matrix(scan("../datasets/SP500",skip=15),ncol=5,byrow=T)
 SP500 <- rev(SP500[1:2529,5])            #closing prices from Jan 2,90
 SP500 <- log(as.numeric(SP500))
 

 ###########  one period return from Jan. 3, 1990   #############
 r1 <- diff(SP500)^2 
 lambda <- 0.97
 xlam <- r1*(1-lambda)
 xlam[1] <- r1[1]
 vol1 <- filter(xlam, filter = lambda, method = "rec")
 vol1 <- as.vector(25*vol1)         #converting one-day volatility
				    #into 25 day volatility

 ########## 25-day return from Jan. 27, 1990   ###############
 r25 <- SP500[26:length(SP500)] - SP500[1:(length(SP500)-25)] 
 r25 <- r25^2
 xlam <- r25*(1-lambda)
 xlam[1] <- r25[1]
 vol25 <- filter(xlam, filter = lambda, method = "rec")
 vol25 <- as.vector(vol25)


 ########## Fitting:  predicting future 25-day volatility from
 ########## vol25 and vol1, starting from Jan. 2, 1991 
 ########## (254 trading-days away Jan. 2, 1990), to April 30, 1996
 ########## (1500 trading away from Jan. 2,90)
 ###################################################################

 y <- r25[253:1475]              #future 25-day squared return
 vola1 <- vol1[252:1474]         #observed 1-day volatility
 vola25 <- vol25[(253-25):(1475-25)] #observed 25-day volatility

 ############ taking a subset to reduce the influence of outliers ###
 sub <- (y < quantile(y,0.975)) + (y > quantile(y,0.025))
 sub <- (1:length(sub))[sub==2]
 ys <- y[sub]
 vola1s  <- vola1[sub]
 vola25s <- vola25[sub]
 res.gam   <- gam(ys ~ lo(vola1s,span=0.75) + lo(vola25s,span=0.75))
 res.term  <- predict(res.gam, type="term")

 ########### predicting volatility in the in-sample period ######
 pred = approx(vola1s, res.term[,1],vola1, rule=3)$y +
        approx(vola25s,res.term[,2],vola25,rule=3)$y + mean(ys)

 tsplot(vola1, ylim=c(0,0.004), xlab="(a)")
 lines(1:length(vola1), abs(pred), lty=2)
 title("Predicted 25-day volatility in the in-sample period")
 y <- SP500[26:length(SP500)] - SP500[1:(length(SP500)-25)]
 y <- y[253:1475] 

 ER   = mean(y < -1.645 * sqrt(abs(pred)))  #Exceedance ratio for
				            #additive model
 ERjp = mean(y < -1.645 * sqrt(vola1))      #Exceedance ratio for
					    #J.P. Morgan method

 print("in-sample period")
 print(paste("ER = ", ER))
 print(paste("ERjp = ", ERjp))

 ############# Prediction, after April 30, 1996######################
 volb1  <- vol1[1500:length(vol1)] 
 volb25 <- vol25[(1501-25):(length(vol25))]
 pred = approx(vola1s, res.term[,1],volb1,rule=3)$y +
        approx(vola25s,res.term[,2],volb25,rule=3)$y + mean(ys)
			   # predicted 25-day volatility
 pred = abs(pred)

 ################## J.P. Morgan method #######################
 yobs <-SP500[1526:length(SP500)] - SP500[1501:(length(SP500)-25)]
 volc1 <- volb1[1:(length(volb1)-25)]
 MSEjp  <- mean((yobs^2 - volc1)^2)
 MADEjp <- mean(abs(yobs^2 - volc1))
 ERjp   <-  mean(yobs < -1.645 * sqrt(volc1)) 
 print("J.P. Morgan method")
 print(paste("MSEjp  = ", MSEjp))
 print(paste("MADEjp = ", MADEjp))
 print(paste("ERjp   = ",   ERjp))


 ################## Additive model method #######################
 pred = pred[1:(length(pred)-25)]
 MSE = mean((yobs^2 - pred)^2)         #Mean square error
 MADE = mean(abs(yobs^2 - pred))       #Mean absolute deviation
 ER   = mean(yobs < -1.645 * sqrt(pred))
 print("Additive model method")
 print(paste("MSE  = ", MSE))
 print(paste("MADE = ", MADE))
 print(paste("ER   = ", ER))

 
 ################# plotting 25-day predicted volatility #################
 tsplot(sqrt(volc1),xlab="(b)", ylim=c(0,0.12))
 lines(1:length(volc1), sqrt(pred),lty=2)
 title("Predicted 25-day volatility")

 pred = pred[1:(length(pred)-25)]
 for(i in 1:length(yobs))
   { if(yobs[i] < 0) 
        lines(c(i,i),c(0, -yobs[i]/1.645))
   }
 abline(0,0) 
}
